/* 
 * File:   customList.h
 * Author: luzem
 *
 * Created on 14 de noviembre de 2009, 19:54
 */
#include "list.h"       // kernel list

#ifndef _CUSTOMLIST_H
#define	_CUSTOMLIST_H

#ifdef	__cplusplus
extern "C" {
#endif


//Struct Declaration
struct customList
    {
    int pid;
    struct list_head list;
    int arrivetime;
    int processtime;
    float waitaverange;
    int stopCounter;
    int lastScheculerActiveTime;
    bool finished;
    };

//struct list_head *first_element=NULL; // used as a mark to point the begin of the tail
struct list_head *iteratorFirstElement=NULL; // used as a mark to point the begin of the tail


void copyInternalValues(struct customList *original, struct customList *copied)
    {
    copied->arrivetime=original->arrivetime;
    copied->finished=original->finished;
    copied->pid=original->pid;
    copied->processtime=original->processtime;
    copied->stopCounter=original->stopCounter;
    }
/*
 * initializeIterator -- Craetes a mark to define the iterator last element
 * firstElement -- the element to be used as iterator mark
 */
void initializeIterador(struct list_head *firstElement)//DONE
    {
    iteratorFirstElement=firstElement;
    }
/*
 * hasNext - check if a iterator has a next element
 * actualElement -- ta pointer of the actual element
 */
bool hasNext(struct list_head *actualElement)//DONE
     {
    bool toret=true;
    if (iteratorFirstElement!=NULL)
        {
    //printf("actualElement->netx== %p first_element==%p\n",actualElement->next,first_element);
        if (actualElement==iteratorFirstElement)
            {
            toret=false;
            }
        }
    return toret;
     }

/*
 * getCustomListPointer -- gets the customlist memory address from a listPointer calculating the offsett
 *  listPointer -- a pointer that point a list element
 */
struct customList  *getCustomListPointer(struct list_head *listPointer)//DONE
    {
    void * toret=NULL;
    toret=listPointer;
    //We use a void pointer because i can't sobrem problems with pointers aritmetic
    toret= (toret - ((unsigned long) &((struct customList *)0)->list));
    return toret;
    }

/*
 * printSelectListElement() -- prints the values of the selected element
 * @listForPrint -- a pointer for the customList to print
 * @identifier   -- a string for identify the custonList <-- for debug
 *
 */
void printSelectListElement(struct customList *listForPrint, char* identifier )//DONE
    {
    printf("**********************************************\n");
    printf("For the list element %s\n",identifier);
    printf("Is stored in the %p address \n",listForPrint);
    printf("The pid value is %d \n",listForPrint->pid);
    printf("List_head is stored in the address %p\n",&listForPrint->list);
    printf("List_head next points to %p\n",listForPrint->list.next);
    printf("List_head prev points to %p\n",listForPrint->list.prev);
    printf("**********************************************\n");
    }


struct customList *getCopyList(struct customList *original) //Unused
    {
    struct customList *toret; //list to return
    struct list_head *iteratorElement;
    //initialize the new list
    toret=NULL;
    initializeIterador(&original->list);
    do{
        //addElement to copiedList
        copyInternalValues(getCustomListPointer(iteratorElement),toret);
        iteratorElement=iteratorElement->next;

        }while(hasNext(iteratorElement));

    return (toret);
    }

void deleteElement(struct list_head *elementToDelete)//Done
    {
    struct customList *toDelete;
    toDelete=getCustomListPointer(elementToDelete);
    list_del(elementToDelete);
    free(toDelete);
    }

/*
 *
 *addNewElement -- Adds a new element into the list
 * pid -- pid of the new element
 * arrivetime -- moment when the process goes into the tail
 * processtime -- time processed
 * stopCounter -- a counter that grows when the process is stoped
 * finished -- a boolean value that indicates if the process has finished
 * list -- the list where the element has added
 *
 */
void addNewElement(int pid,int arrivetime,int processtime, int stopCounter,bool finished,struct customList **list)
    {
    if (*list==NULL)//List not initialited
        {
        *list=(struct customList *) malloc(sizeof(struct customList));//reserve space in the memory for the new element
        (*list)->pid=pid;//assing a new pid number
        (*list)->arrivetime=arrivetime;
        (*list)->processtime=processtime;
        (*list)->stopCounter=stopCounter;
        (*list)->finished=finished;
        (*list)->lastScheculerActiveTime=0;
        (*list)->waitaverange=0;
        INIT_LIST_HEAD(&(*list)->list);
        //printf("Custom list points to %p\n",*list);
        }
    else //exist elements in the list
        {
        //printf("Not the first");
        struct customList *newElement;//create the new element
        newElement = (struct customList *) malloc(sizeof(struct customList));//reserve space in the memory for the new element
        newElement->pid=pid;//assing a new pid number
        newElement->arrivetime=arrivetime;
        newElement->processtime=processtime;
        newElement->stopCounter=stopCounter;
        newElement->finished=finished;
        list_add(&newElement->list,&(*list)->list);
        }
    }

void copyCustomList(struct list_head *source, struct customList **destination,struct list_head *firstElement)
    {
    struct customList *toCopy=NULL;
    toCopy=getCustomListPointer(source);
    if (source->next==firstElement)//We've reached the last element
        {
        //printf("Last Element\n");
        addNewElement(toCopy->pid,toCopy->arrivetime,toCopy->processtime,toCopy->stopCounter,toCopy->finished,&(*destination));
        //printf("Last Element Added\n");
        }
    else
        {
        copyCustomList(source->next,destination,firstElement);
        //printf("Adding Element\n");
        addNewElement(toCopy->pid,toCopy->arrivetime,toCopy->processtime,toCopy->stopCounter,toCopy->finished,&(*destination));
        //printf("Element Added\n");
        }
    }

void EmptyCustomList(struct list_head *firstElement,struct list_head *nextElement)
    {
    if ((nextElement)==NULL)//Initialites the deletion
        {
        //printf("First Intro\n");
        EmptyCustomList(firstElement,firstElement->next);

        }
    else
        {
        if (nextElement==firstElement) //We've reached the last element in the list
            {
            //printf("Ending Deletion of last Element\n");
            deleteElement(nextElement);
            //printf("Last Element Deleted\n");
            }
        else
            {
            //printf("Deleting %p element\n",nextElement);
            EmptyCustomList(firstElement,nextElement->next);//Move next Element
            //printf("Deleted %p element\n",nextElement);
            deleteElement(nextElement);
            }

        }
    }

/*
 * deleteByPid -- removes a element in the custom list checking the pid
 * pid -- the pid value of the element to be removed
 */
bool deleteByPid(int pid, struct list_head *firstElement)
    {
    bool toret=false;
    struct list_head *iterator;
    struct customList *actualElement;
    iterator=firstElement;
    //printf("Initializing iterator\n");
    initializeIterador(firstElement);
    //printf("Iterador initialized\n");
    do{
       // printf("Calling get Custom List Pointer\n");
        actualElement=getCustomListPointer(iterator);
        //printSelectListElement(actualElement,"");
        //printf("Comparing pid\n");
        if (actualElement->pid==pid)
            {
            //printf("Deleting Element \n");
            deleteElement(iterator);
            toret=true;
            }
        iterator=iterator->next;
        }while(hasNext(iterator));
    return toret;
    }



int getElementsCount(struct customList *list)
    {
    int toret=0;
    struct list_head *iterator;
    struct list_head *actualElement;
    iterator=&list->list;
    actualElement=iterator->next;
    while (iterator!=actualElement)
        {
        toret++;
        actualElement=actualElement->next;
        }
    return toret;
    }

void printLine(struct customList *process, int timeslice, int contextchangue)
    {
    /*
     *     int pid;
    struct list_head list;
    int arrivetime;
    float processtime;
    float waitaverange;
    int stopCounter;
    bool finished;
     * */
    printf("%d\t,%d\t,%d\t,%d\t,%f\t,%d\n",timeslice,contextchangue,process->pid,(process->stopCounter*timeslice)-process->processtime,process->waitaverange, process->stopCounter );
    }

void printCVSHeader()
    {

    printf("Timeslice,ContextChangue,PID,TotalTime,WaitAverange,StopCounter\n");
    }
#ifdef	__cplusplus
}
#endif

#endif	/* _CUSTOMLIST_H */